# **Report Issues**
Any problem is encouraged to be published when you participate in MatrixOne, and you can [report issues](https://github.com/matrixorigin/matrixone/issues/new/choose) about this problem in GitHub with relative templates and labels we suggest.  
This page is intended to introduce templates, labels, and precautions you will follow when you report issues.

## **Avoid Duplicated Issues**
Before reporting an issue, you should see whether the problem has already been reported. You can use the [search bar](https://docs.github.com/en/issues/tracking-your-work-with-issues/filtering-and-searching-issues-and-pull-requests) to search existing issues.

## **Issue Templates**
If the problem you're reporting is not already in the issue tracker, you can file an issue with your GitHub account. MatrixOne uses issue templates for different kinds of issues. Issue templates are a bundle of questions to collect necessary information about the problem to make it easy for other contributors to participate.
MatrixOne suggests 6 types of issue templates: 

A `Bug report` issue template consists of information below:  

* **Detail Environment**  
  Describe the details about the operating environment including version, hardware parameters, OS type and so on.
* **Steps to Reproduce**  
  List steps to reproduce what you encountered.
* **Actual & Expected Behavior**  
  Describe what's the observed and your expected behavior respectively.

A `Feature Request` consists of information below:  

* **Is your feature request related to a problem?**  
A clear and concise description of what the problem is and state your reasons why you need this feature.
* **Describe the feature you'd like:**  
A clear and concise description of what you want to happen.
* **Describe alternatives you've considered:**  
A clear and concise description of any alternative solutions or features you've considered. 
* **Teachability, Documentation, Adoption, Migration Strategy:**  
If you can, explain some scenarios how users might use this, situations it would be helpful in. Any API designs, mockups, or diagrams are also helpful. 

A `Documentation Issue` consists of information below:  

* **Describe the issue**  
  A clear and concise description of what's wrong in documentation.
* **Additional context**  
  Add any other context about the problem here.


A `Performance Question` consists of information below:  

* **Detail Environment**  
  Describe the details about the operation environment including version, hardware parameters, OS type and so on.
* **Steps to Reproduce**  
  List steps detailedly to reproduce the operations to test performance.
* **Expected & Actual Performance**  
  Describe what's the observed and your expected performance respectively.
* **Additional context**  
  Add any other context about the problem here. For example:  
    * Have you compared MatrixOne with other databases? If yes, what's their difference?

An `Enhancement` consists of information below:  

* **What would you like to be added**  
A concise description of what you're expecting/suggesting.
* **Why is this needed**  
A concise description of the reason/motivation.
* **Anything else**  
Anything that will give us more dectail about your issue!


A `Refactoring Request` consists of information below:  

* **Is your refactoring request related to a problem?**  
A clear and concise description of what the problem is.
* **Describe the solution you'd like**  
A clear and concise description of the refactoring you want to.
* **Describe alternatives you've considered**  
A clear and concise description of any alternative solutions or refactoring method you've considered.
* **Additional context**  
Add any other context or screenshots about the refactoring request here.

A `General Question`:

If the previous templates don't fit with what you'd like to report or ask, please use this general question template to file issue. 

Before asking a question, make sure you have:

- Searched open and closed [GitHub issues](https://github.com/matrixorigin/matrixone/issues)
- Read the documentation:
  - [MatrixOne Readme](https://github.com/matrixorigin/matrixone)
  - [MatrixOne Doc](https://docs.matrixorigin.io/)


## **Issue Labels**

Once your issue is submitted, it will be automatically labeled with `needs-triage`, a MatrixOne maintainer will look at the issue, tag it with more suitable labels and assign it to a suitable developer. 
If you'd like to work on the issue by yourself, you can comment `/assign` on the issue you would like to work on, you will be automatically assigned to the issue. You can then find yourself listed under Assignees section.

## **Good First Issues**
When you make the first contribution to MatrixOne, you can select one of the issues labeled [`good-first-issues`](https://github.com/matrixorigin/matrixone/issues?q=is%3Aopen+is%3Aissue+label%3A%22good+first+issue%22) each of which is relatively approachable for first-time contributors.
For detailed suggestions you need when you make the first contribution, you can see [Make Your First Contribution](make-your-first-contribution.md). 

